#!/usr/bin/perl
# Show a form for registering a xen host, and editing config settings

require './server-manager-lib.pl';
&can_action_error(undef, "global");
&ReadParse();
&ui_print_header(undef, $in{'new'} ? $text{'xen_create'} :
		        $module_info{'onehost'} ? $text{'xen_editone'} :
						  $text{'xen_edit'}, "",
		 $in{'new'} ? "createxen" : "editxen");
@allservers = &list_managed_servers();
($xen) = grep { $_->{'id'} eq $in{'id'} } @allservers;
$xen || &error($text{'xen_egone'});

if ($in{'new'}) {
	# Can this system even host Xen instances?
	$err = &check_xen_host_software($xen);
	if ($err) {
		print &ui_confirmation_form(
			"install_xen.cgi",
			&text('xen_einstall', $err),
			[ [ 'id', $in{'id'} ] ],
			[ [ undef, $text{'xen_install'} ] ],
			);
		&ui_print_footer("list_xens.cgi", $text{'xens_return'});
		return;
		}
	}

print &ui_form_start("save_xen.cgi", "post");
print &ui_hidden("new", $in{'new'});
print &ui_hidden_table_start($text{'xen_header'}, "width=100%", 2, "main", 1);

# System to register
print &ui_hidden("id", $in{'id'});
if (!$module_info{'onehost'}) {
	print &ui_table_row($text{'xen_host'},
			    $xen->{'host'});
	}

if (!$module_info{'onehost'}) {
	# Location group
	&show_host_location_field($xen);
	}

# Xen directory
print &ui_table_row($text{'xen_dir'},
	&ui_textbox("dir", $xen->{'xen_dir'} || "/xen", 60));

# Find volume groups on host, if we can
@pvs = ( );
if ($xen->{'status'} ne 'down' &&
    &remote_foreign_check($xen, "lvm")) {
	$last_remote_error = undef;
	&remote_foreign_require($xen, "lvm", "lvm-lib.pl");
	if (!$last_remote_error) {
		@pvs = &remote_foreign_call($xen, "lvm",
					    "list_volume_groups");
		$no_lvm = 1 if (!@pvs && !$last_remote_error);
		}
	}

# Use drop-down of VG if possible, text field otherwise
if (@pvs) {
	$lvfield = &ui_select("lvm", $xen->{'xen_vg'},
			      [ map { $_->{'name'} } @pvs ], 1, 0,
			      $xen->{'xen_vg'} ? 1 : 0);
	}
else {
	$lvfield = &ui_textbox("lvm", $xen->{'xen_vg'}, 40);
	}

# Check if iSCSI is supported
@iservers = &list_iscsi_target_servers();
if (&supports_iscsi() && @iservers) {
        $ifield = &ui_select("iscsi", $xen->{'xen_iscsi'},
                     [ map { [ $_->{'id'}, $_->{'host'} ] } @iservers ]);
        }

# Build table of storage types
@stable = ( [ 1, $text{'kvm_storage1'} ] );
if (!$no_lvm) {
        push(@stable, [ 0, $text{'kvm_storage0'}, $lvfield ]);
        }
if ($ifield) {
        push(@stable, [ 2, $text{'kvm_storage2'}, $ifield ]);
        }
if (@stable > 1) {
        print &ui_table_row($text{'kvm_storage'},
                &ui_radio_table("lvm_def", $xen->{'xen_iscsi'} ne '' ? 2 :
                                           $xen->{'xen_vg'} ? 0 : 1, \@stable));
        }

# DNS domain name to add under (if we host any)
@zones = &list_bind_zones();
$zonename = $xen->{'xen_zone'} || $config{'xen_zone'};
$link = " ".&bind_module_link();
if (@zones) {
	print &ui_table_row($text{'xen_domain'},
		&ui_select("zone", $zonename,
		    [ map { [ $_->{'name'}, $_->{'desc'} ] } @zones ]).$link);
	}
else {
	print &ui_table_row($text{'xen_domain2'},
		&ui_textbox("zone", $zonename, 30).$link);
	}

# Xen file driver type
$xen->{'xen_filetype'} = 'file' if (!$xen->{'xen_filetype'});
print &ui_table_row($text{'xen_filetype'},
	&ui_radio("filetype", $xen->{'xen_filetype'},
		  [ $in{'new'} ? ( [ "", $text{'xen_ft_auto'} ] ) : ( ),
		    [ "file", $text{'xen_ft_file'} ],
		    [ "tap:aio", $text{'xen_ft_aio'} ] ]));

if (!$module_info{'onehost'}) {
	# Live migration IP and port
	print &ui_table_row($text{'xen_mighost'},
		&ui_opt_textbox("livehost", $xen->{'xen_livehost'},
			40, $text{'xen_mighost1'}, $text{'xen_mighost0'}));
	print &ui_table_row($text{'xen_migport'},
		&ui_opt_textbox("liveport", $xen->{'xen_liveport'},
			10, $text{'default'}." (".$xen_migration_port.")",
			$text{'xen_migport0'}));
	}

# Add all existing Xen instances at same time?
if ($in{'new'}) {
	# Auto-detect?
	print &ui_table_row($text{'xen_autoadd'},
		&ui_radio("autoadd", 0,
			  [ [ 0, $text{'no'} ],
			    [ 1, $text{'xen_autoadd1'} ],
			    [ 2, $text{'xen_autoadd2'} ] ]));

	# Login type
	print &ui_table_row($text{'xen_autologin'},
		&ui_radio("autologin", 1,
			  [ [ 0, $text{'xen_autologin0'} ],
			    [ 1, $text{'xen_autologin1'} ] ]));
	}

# Xen version
if (&can_ssh_status($xen)) {
	my $ver = &get_xen_software_version($xen, 1);
	if ($ver) {
		print &ui_table_row($text{'xen_version'}, $ver);
		}
	}

print &ui_hidden_table_end("main");

print &ui_hidden_table_start($text{'xen_nheader'}, "width=100%", 2, "net", 0);

# IP allocation range and CIDR
@possifaces = &list_host_bridges($xen);
@hostifaces = split(/\s+/, $xen->{'xen_iface'});
@allocifaces = &unique(@possifaces, @hostifaces);
$multi_ifaces = @possifaces > 1 || $xen->{'xen_iface'};
print &ui_table_row($text{'xen_range'},
	&show_ip_range_input($xen, "xen", 1,
			     $multi_ifaces ? \@allocifaces : undef));

# Show IPv6 allocation ranges, if any
$input6 = &show_ip6_range_input($xen, "xen", 1,
				$multi_ifaces ? \@allocifaces : undef);
print &ui_table_row($text{'kvm_range6'}, $input6) if ($input6);

# Network interfaces to use on host system
if ($multi_ifaces) {
	print &ui_table_row($text{'xen_iface'},
		&ui_radio("iface_def", scalar(@hostifaces) ? 0 : 1,
			  [ [ 1, $text{'xen_iface1'} ],
			    [ 0, $text{'xen_iface0'} ] ])."<br>\n".
		&ui_select("iface", \@hostifaces, \@possifaces,
			   scalar(@possifaces), 1, 1));
	}

# Source resolv.conf file
print &ui_table_row($text{'xen_resolv'},
	&ui_opt_textbox("resolv", $xen->{'xen_resolv'}, 30,
			$text{'xen_resolvdef'}, $text{'xen_resolvother'})." ".
	&file_chooser_button("resolv"));

print &ui_hidden_table_end("net");

# Section for minimum resources
&show_host_floor_table($xen);

if (!$module_info{'onehost'}) {
	# Section for caching
	&show_host_images_cache_table($xen, "xen");
	}

# Section for default backup destination
&show_default_backup_dest_table($xen);

print &ui_form_end($in{'new'} ? [ [ undef, $text{'xen_reg'} ] ]
			      : [ [ undef, $text{'save'} ],
				  [ "delete", $text{'xen_unreg'} ] ]);

if ($module_info{'onehost'}) {
	&ui_print_footer("", $text{'index_return'});
	}
else {
	&ui_print_footer("list_xens.cgi", $text{'xens_return'});
	}

