#!/usr/bin/perl

=head1 setup-xen-system.pl

Configures a physical system as a Xen host.

This command installs and configures the packages needed for hosting Xen
virtual machines on a physical system. It is primarily designed to be called
from the Cloudmin installer, but can be used from the shell as well.

=head1 Example usage

  cloudmin setup-xen-system --host xenhost.home
Setting up Xen host system ..
    Checking system status ..
    .. done

    Installing required commands ..
    .. done

    Setting up Xen kernel repository ..
    .. done

    Installing Xen kernel and tools ..
    .. done

    Removing virbr network interfaces ..
    .. none found

    Setting up xenbr0 network bridge ..
    .. done

.. setup complete, but a reboot is required. You can then add this system as a Xen host.

=cut

package server_manager;
$main::no_acl_check++;
$ENV{'WEBMIN_CONFIG'} ||= "/etc/webmin";
$ENV{'WEBMIN_VAR'} ||= "/var/webmin";
if ($0 =~ /^(.*)\/[^\/]+$/) {
	chdir($pwd = $1);
	}
else {
	chop($pwd = `pwd`);
	}
$0 = "$pwd/setup-xen-system.pl";
require './server-manager-lib.pl';
$< == 0 || die "$0 must be run as root";
@OLDARGV = @ARGV;

# Parse command line
while(@ARGV > 0) {
	my $a = shift(@ARGV);
	if ($a eq "--host") {
		$host = shift(@ARGV);
		}
	elsif ($a eq "--multiline") {
		$multiline = 1;
		}
	else {
		&usage("Unknown parameter $a");
		}
	}

# Get the server and check parameters
$host || &usage("Missing --host parameter");
$server = &find_one_managed_server($host);
$server || &usage("No system with hostname $host exists");
&can_action_usage(undef, "global");

# Install as Xen host
&set_all_text_print();
&$first_print($text{'xeninstall_doing'});
&$indent_print();
$ok = &install_xen_host_software($server);
&$outdent_print();

# Evaluate the output
if ($ok) {
	&$second_print($ok == 2 ? $text{'xeninstall_reboot'}
				: $text{'xeninstall_done'});
	&cloudmin_api_log(\@OLDARGV);
	}
else {
	&$second_print($text{'xeninstall_failed'});
	exit(1);
	}

sub usage
{
print "$_[0]\n\n" if ($_[0]);
print "Configures a physical system as a Xen host.\n";
print "\n";
print "cloudmin setup-xen-system --host name\n";
print "                         [--qemu]\n";
exit(1);
}


