# Functions for managing the server itself

sub type_this_desc
{
return $text{'this_typedesc'};
}

sub type_this_no_shutdown
{
return 1;	# Don't let the system shut itself down
}

sub type_this_no_reboot
{
return 1;	# Don't let the system reboot itself
}

sub type_this_no_startup
{
return 1;	# This system cannot start itself up!
}

sub type_this_no_ssh
{
return 1;	# We never need to SSH into vservers
}

sub type_this_no_image
{
return 1;	# Real systems cannot be imaged
}

sub type_this_can_transfer
{
return 1;	# Can always transfer to self
}

sub type_this_can_password
{
return 1;
}

# type_this_execute_command(&server, comand, [timeout], [leave-open])
# Just runs a command locally
sub type_this_execute_command
{
my ($server, $cmd, $timeout, $leave) = @_;
if ($leave) {
	# Open the command in a new pty
	&foreign_require("proc", "proc-lib.pl");
	&clean_environment();
	my ($fh, $fpid) = &proc::pty_process_exec($cmd);
	&reset_environment();
	return (undef, 0, undef, 0, $fh, $fpid);
	}
else {
	# Just run the command, and get the output
	&clean_environment();
	if ($cmd !~ /^\s*\(.*\)\s*$/) {
		$cmd = "(".$cmd.")";
		}
	my $out = $timeout ?
		&backquote_with_timeout("$cmd 2>&1 </dev/null", $timeout) :
		&backquote_command("$cmd 2>&1 </dev/null");
	&reset_environment();
	return ($out, $?/256);
	}
}

# type_this_get_file(&server, remotefile, localfile, device-file)
# Can just copy the file, since we are on the same box!
sub type_this_get_file
{
my ($server, $remotefile, $localfile, $device) = @_;
my $out = &backquote_command(
	$device ?
		&dd_command($remotefile, $localfile)." 2>&1" :
	-d $localfile && -d remotefile ?
		"cp -r -H ".quotemeta($remotefile)."/* ".
		  quotemeta($localfile)." 2>&1" :
		"cp -r -H ".quotemeta($remotefile)." ".
		  quotemeta($localfile)." 2>&1");
return $? ? $out : undef;
}

# type_this_put_file(&server, localfile, remotefile, device-file)
# Can just copy the file, since we are on the same box!
sub type_this_put_file
{
my ($server, $localfile, $remotefile, $device) = @_;
return &type_this_get_file($server, $localfile, $remotefile, $device);
}

sub type_this_check_login
{
my ($server) = @_;
my %miniserv;
&get_miniserv_config(\%miniserv);
return (0, undef, $miniserv{'root'}, $miniserv{'port'}, $miniserv{'ssl'});
}

# type_this_stat_file(&server, remotefile, [lookup-usernames])
# For a file on the Cloudmin master, just stat it directly
sub type_this_stat_file
{
my ($server, $remotefile, $usernames) = @_;
my @st = lstat($remotefile);
return () if (!@st);
$st[13] = $remotefile;
$st[14] = readlink($remotefile);
if ($usernames) {
	$st[4] = getpwuid($st[4]) || $st[4];
	$st[5] = getgrgid($st[5]) || $st[5];
	}
return @st;
}

# type_this_set_ownership_permissions(&server, user, group, perms, [file]+)
# Set permissions on a local file
sub type_this_set_ownership_permissions
{
my ($server, $user, $group, $perms, @files) = @_;
return &set_ownership_permissions($user, $group, $perms, @files) ? undef : "$!";
}

# type_this_rename_file(&server, oldfile, newfile)
# Return a file or directory
sub type_this_rename_file
{
my ($server, $oldfile, $newfile) = @_;
return &rename_file($oldfile, $newfile) ? undef : "Rename failed";
}

# type_this_make_dir(&server, remotedir, perms, recur)
# For a dir on the Cloudmin master, just create it directly
sub type_this_make_dir
{
my ($server, $remotedir, $perms, $recur) = @_;
my $ok = &make_dir($remotedir, $perms, $recur);
return $ok ? undef : "$!";
}

# type_this_list_dir(&server, remotedir, [full-stat], [lookup-usernames],
# 		     [remove-dir])
# For a dir on the Cloudmin master, just list it directly
sub type_this_list_dir
{
my ($server, $remotedir, $fullstat, $usernames, $removedir) = @_;
opendir(LISTDIR, $remotedir) || return "$!";
my @files = readdir(LISTDIR);
if ($fullstat) {
	my $d = $remotedir eq "/" ? "" : $remotedir;
	$d =~ s/\/$//;
	@files = map { [ &type_this_stat_file($server, $d."/".$_, $usernames) ]
		     } @files;
	foreach my $st (@files) {
		$st->[13] =~ s/^\Q$removedir\E// if ($removedir);
		}
	}
closedir(LISTDIR);
return \@files;
}

# type_this_readlink(&server, file)
# For a file on the Cloudmin master, just check it directly
sub type_this_readlink
{
my ($server, $file) = @_;
return readlink($file);
}

# type_this_symlink_file(&server, source, dest)
# Just call local link function
sub type_this_symlink_file
{
my ($server, $src, $dest) = @_;
return &symlink_file($src, $dest) ? undef : $!;
}

# type_this_open_shell(&server)
# Opens a shell in a sub-process, connected to a pty. Returns an error message
# (or undef if OK), and a file handle.
sub type_this_open_shell
{
my ($server) = @_;
&foreign_require("proc");
my $shell = &has_command("bash") || &has_command("sh");
my $pwd = &get_current_dir();
chdir("/root") || chdir("/");
my ($fh, $pid) = &proc::pty_process_exec($shell);
chdir($pwd);
return (undef, $fh);
}

# type_this_real_interfaces(&server)
# Returns a list of interfaces on the master system. Each is a hash ref with
# keys name, bytes_in, bytes_out, packets_in, packets_out
sub type_this_real_interfaces
{
my ($server) = @_;
my $out = &read_file_contents("/proc/net/dev");
my @rv;
foreach my $dev (split(/\r?\n/, $out)) {
	$dev =~ s/^\s+//;
        my @w = split(/[ \t:]+/, $dev);
	if ($w[0] =~ /^eth\d+/) {
		my $iface = { 'name' => $w[0],
			      'bytes_in' => $w[1],
			      'packets_in' => $w[2],
			      'bytes_out' => $w[9],
			      'packets_out' => $w[10] };
		push(@rv, $iface);
		}
	}
return \@rv;
}

1;


